//Copyright (C) 2005  Iain Sharp
//Portions Copyright (C) 1994  Scott Draves <spot@cs.cmu.edu>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//Linking Swell statically or dynamically with other modules is making a
//combined work based on Swell.  Thus, the terms and conditions of the GNU
//General Public License cover the whole combination.
//
//As a special exception, the copyright holders of Swell give you
//permission to integrate Swell and derivative works 
//with multimedia libraries and toolkits similar to
//DirectX and PocketHAL, regardless of the license
//terms of these independent modules, and to copy and distribute the
//resulting combined work under the GPL maintaining this exception.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// For information and contacts for the author go to www.pinedragon.com/swell
//


#include <windows.h>
//#include <io.h>
//#include <stdio.h>
//#include <FCNTL.H>
#include "defs.h"
#include "platformdefs.h"
#include "resource.h"

cmap_t g_ithecmaps[MAXCMAPS];
int g_incmaps=0;
mypixel_t g_aMyCmap[256];
mypixel_t g_aTargetCmap[256];
mypixel_t g_aSliderCmap[256];


void loadbincmaps();

#ifndef _WIN32_WCE
DWORD rgbtocmap(BYTE r, BYTE g, BYTE b)
{
	return (DWORD)RGB(b,g,r);
}

#else
DWORD rgbtocmap(BYTE r, BYTE g, BYTE b)
{
	return (((r & 0xf8) << 8) | ((g &0xfc) << 3) | (b>>3));
}

#endif

int round_up8(int n)
{
	int m;

   if (n < 0)
      m = (n - 7) >> 3;
   else
      m = (n + 7) >> 3;
#ifdef _WIN32_WCE
	if ((m>-8)&&(m<8))
		return n;
	else
		return m;
#else
	return m;
#endif
}

void makemap(int i)
{
	int x;
	cmap_t* pm;
	pm=&g_ithecmaps[i];
	for (x=0;x<256;x++)
	{
		g_aMyCmap[x]=rgbtocmap((*pm)[x][0],(*pm)[x][1],(*pm)[x][2]);
		g_aTargetCmap[x]=g_aMyCmap[x];
		g_aSliderCmap[x]=g_aMyCmap[x];
	}
	return;
}

void maketargetmap(int i)
{
	int x;
	cmap_t* pm;
	pm=&g_ithecmaps[i];
	for (x=0;x<256;x++)
	{
		g_aTargetCmap[x]=rgbtocmap((*pm)[x][0],(*pm)[x][1],(*pm)[x][2]);
	}
	return;
}


bool iteratecmap()
{
	int i,r,g,b;
	bool diff=false;
	for(i=0;i<256;i++)
		if (g_aTargetCmap[i]!=g_aMyCmap[i])
		{
			diff=true;
			r=RG(g_aMyCmap[i])+round_up8(RG(g_aTargetCmap[i]) - RG(g_aMyCmap[i]) );
			g=GG(g_aMyCmap[i])+round_up8(GG(g_aTargetCmap[i]) - GG(g_aMyCmap[i]) );
			b=BG(g_aMyCmap[i])+round_up8(BG(g_aTargetCmap[i]) - BG(g_aMyCmap[i]) );
			g_aMyCmap[i]=rgbtocmap(r,g,b);
		}
	return diff;
}

void makeslidermap(int i)
{
	int x;
	cmap_t* pm;
	pm=&g_ithecmaps[i];
	for (x=0;x<256;x++)
	{
		g_aSliderCmap[x]=rgbtocmap((*pm)[x][0],(*pm)[x][1],(*pm)[x][2]);
	}
	return;
}


void initcmap()
{
	
	int x;
	cmap_t* pm;
	pm=&g_ithecmaps[0];
	// black -> midred -> red -> yellow -> white
	// NB - order of params to RGB is wrong - hack to fix difference between COLORSPEC and Bitmap formats!
      for (x = 0; x < 128; x++) {
	 (*pm)[x+128][0]= (*pm)[x+128][1]= 255;
	 (*pm)[x+128][2] = x << 1;
      }
      for (x = 0; x < 64 ;x++) {
	 (*pm)[x+64][0] = 255;
	 (*pm)[x+64][2] = 0;
	 (*pm)[x+64][1] = x<<2;
      }
      for(x = 0; x < 64; x++) {
	 (*pm)[x][0] = x<<2;
	 (*pm)[x][1]=(*pm)[x][2] = 0;
      }
	g_incmaps=1;
	loadbincmaps();
	makemap(0);
}
void ramp(int s,int d, int i0, int i1, int n, int k)
{
   int x;
   for (x = 0; x < n; x++) {
      double alpha = x / (double) n;
      g_ithecmaps[d][k + x][0] = (int) (g_ithecmaps[s][i0][0] * (1 - alpha) +
											g_ithecmaps[s][i1][0] * alpha);
      g_ithecmaps[d][k + x][1] = (int) (g_ithecmaps[s][i0][1] * (1 - alpha) +
											g_ithecmaps[s][i1][1] * alpha);
      g_ithecmaps[d][k + x][2] = (int) (g_ithecmaps[s][i0][2] * (1 - alpha) +
											g_ithecmaps[s][i1][2] * alpha);
   }
}

int next_contrasting_color(int c, int i, int d)
{
   int nchecked = 0;
   int j = i;
   int dd;
   int max_i = 0;
   int max = 0;
   while (nchecked++ < 250) {
      int t;
      j = (j+1)%256;

	  t = (g_ithecmaps[c][i][0] - g_ithecmaps[c][j][0]);
      dd = t * t;
      t = (g_ithecmaps[c][i][1] - g_ithecmaps[c][j][1]);
      dd += t * t;
      t = (g_ithecmaps[c][i][2] - g_ithecmaps[c][j][2]);
      dd += t * t;

      if (dd > max) {
	 max_i = j;
	 max = dd;
      }
      if (dd >= d)
	 return j;
   }
   return max_i;
}

void makeloopmap(int s,int d)
{
	int i0,i1,i2;
      i0 = rand()%256;
      i1 = next_contrasting_color(s, i0, 60000);
      i2 = next_contrasting_color(s, i1, 60000);
      ramp(s,d, i0, i1, 64, 0);
      ramp(s,d, i1, i2, 64, 64);
      ramp(s, d,i2, i1, 64, 128);
      ramp(s, d,i1, i0, 64, 192);
}

void makepathmap(int s,int d)
{
	int i,i0,i1,i2;
	int n = 10;
	 int len = 30000;
	 int each = 256/(n+1);
	 i2 = i0 = rand()%256;
	 for (i = 0; i < n; i++) {
	    i1 = next_contrasting_color(s, i0, len);
	    ramp(s,d, i0, i1, each, i*each);
	    i0 = i1;
	 }
	 ramp(s,d, i1, i2, 256 - n*each, n*each);
}


void loadbincmaps()
{
	HGLOBAL hresource;
	BYTE * pbytes;
	int i,j;
	int rescmaps;
	hresource=FindResource(g_hinst,MAKEINTRESOURCE(IDR_MYCMAP2),TEXT("MYCMAP"));
	rescmaps=SizeofResource(g_hinst,(HRSRC)hresource)/(256*3);
	hresource=LoadResource(g_hinst,(HRSRC)hresource);
	pbytes=(BYTE *) LockResource(hresource);
	for (i=1;i<=rescmaps;i++)
	{
		for(j=0;j<256;j++)
		{
				g_ithecmaps[g_incmaps][j][0]=*pbytes++;
				g_ithecmaps[g_incmaps][j][1]=*pbytes++;
				g_ithecmaps[g_incmaps][j][2]=*pbytes++;
		}
		g_incmaps++;
		if (g_incmaps>=MAXCMAPS)
			break;
		makeloopmap(g_incmaps-1,g_incmaps);
		g_incmaps++;
		if (g_incmaps>=MAXCMAPS)
			break;
//		makepathmap(g_incmaps-1,g_incmaps);
//		g_incmaps++;
//		if (g_incmaps>=MAXCMAPS)
//			break;
	}// for i 
#ifndef _WIN32_WCE
	FreeResource(hresource);
#endif
}



//void loadcmapsandwrite()
//{
//	HGLOBAL hresource;
//	char * ptext;
//	int i,j;
//	int fhandle;
//	BYTE triplet[3];
//
//	hresource=FindResource(g_hinst,MAKEINTRESOURCE(IDR_MYCMAP1),TEXT("MYCMAP"));
//	hresource=LoadResource(g_hinst,(HRSRC)hresource);
//	ptext=(char *) LockResource(hresource);
//	fhandle=_open("bincmap.bin",_O_WRONLY | _O_CREAT | _O_BINARY);
//	while (*ptext!='$')
//	{
//		ptext=(char*)memchr(ptext,10,100)+1;
//		ptext=(char*)memchr(ptext,10,100)+1;
//		for(i=0;i<64;i++)
//		{	
//			for(j=0;j<4;j++)
//			{
//				int *entry = &g_ithecmaps[g_incmaps][i*4+j][0];
//				sscanf(ptext, " ( %d %d %d )", entry, entry+1, entry+2);
//				triplet[0]=*entry;
//				triplet[1]=*(entry+1);
//				triplet[2]=*(entry+2);
//				_write(fhandle,triplet,3);
//			}
//			ptext=(char*)memchr(ptext,10,100)+1;
//		}
//		ptext=(char*)memchr(ptext,10,100)+1;
//		g_incmaps++;
//		if (g_incmaps>=MAXCMAPS)
//			break;
//		makeloopmap(g_incmaps-1,g_incmaps);
//		g_incmaps++;
//		if (g_incmaps>=MAXCMAPS)
//			break;
////		makepathmap(g_incmaps-1,g_incmaps);
////		g_incmaps++;
////		if (g_incmaps>=MAXCMAPS)
////			break;
//	}// while 
//	_close(fhandle);
//#ifndef _WIN32_WCE
//	FreeResource(hresource);
//#endif
//}
//
//void loadcmaps()
//{
//	HGLOBAL hresource;
//	char * ptext;
//	int i,j;
//	hresource=FindResource(g_hinst,MAKEINTRESOURCE(IDR_MYCMAP1),TEXT("MYCMAP"));
//	hresource=LoadResource(g_hinst,(HRSRC)hresource);
//	ptext=(char *) LockResource(hresource);
//	while (*ptext!='$')
//	{
//		ptext=(char*)memchr(ptext,10,100)+1;
//		ptext=(char*)memchr(ptext,10,100)+1;
//		for(i=0;i<64;i++)
//		{	
//			for(j=0;j<4;j++)
//			{
//				int *entry = &g_ithecmaps[g_incmaps][i*4+j][0];
//				sscanf(ptext, " ( %d %d %d )", entry, entry+1, entry+2);
//			}
//			ptext=(char*)memchr(ptext,10,100)+1;
//		}
//		ptext=(char*)memchr(ptext,10,100)+1;
//		g_incmaps++;
//		if (g_incmaps>=MAXCMAPS)
//			break;
//		makeloopmap(g_incmaps-1,g_incmaps);
//		g_incmaps++;
//		if (g_incmaps>=MAXCMAPS)
//			break;
////		makepathmap(g_incmaps-1,g_incmaps);
////		g_incmaps++;
////		if (g_incmaps>=MAXCMAPS)
////			break;
//	}// while 
//#ifndef _WIN32_WCE
//	FreeResource(hresource);
//#endif
//}