
//Copyright (C) 2005  Iain Sharp
//Portions Copyright (C) 1994  Scott Draves <spot@cs.cmu.edu>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//Linking Swell statically or dynamically with other modules is making a
//combined work based on Swell.  Thus, the terms and conditions of the GNU
//General Public License cover the whole combination.
//
//As a special exception, the copyright holders of Swell give you
//permission to integrate Swell and derivative works 
//with multimedia libraries and toolkits similar to
//DirectX and PocketHAL, regardless of the license
//terms of these independent modules, and to copy and distribute the
//resulting combined work under the GPL maintaining this exception.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// For information and contacts for the author go to www.pinedragon.com/swell
//
#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x500

#include <math.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "defs.h"
#include "platformdefs.h"
#include "resource.h"
#include "iconbar.h"
#include "slider.h"

static WNDCLASS iconbarwc;
CIconBar g_aIconBar[NIBS];
int g_inexticonbar=FIRSTIBID;

#ifndef max
	#define max(x,y) ((x) > (y) ? (x) : (y))
#endif



CIconBar::CIconBar(){
	m_hwnd=NULL;
	m_inicons=0;
}

CIconBar::~CIconBar(){
}


void CIconBar::AddIconBar(HWND hwnd)
{
	m_hwnd=hwnd;
}

void CIconBar::AddIcon(int iconnumber, int iconid,int icontype,bool update)
{
	int i;
	m_inicons=max(m_inicons,iconnumber+1);
	m_astate[iconnumber]=INORMAL;
	m_aiconid[iconnumber]=iconid;
	m_aicontype[iconnumber]=icontype;
	for(i=0;i<INICONSTATES;i++)
		m_aiconhandles[iconnumber][i]=LoadIcon(g_hinst,MAKEINTRESOURCE(iconid+i));
	if (update)
	{
			InvalidateRect(m_hwnd,NULL,true);
		UpdateWindow(m_hwnd);
	}
}



void CIconBar::Paint(HDC hdc){
	RECT r;
	int i;

	GetClientRect(m_hwnd,&r);
	SelectObject(hdc,GetStockObject(BLACK_PEN));
	SelectObject(hdc,GetStockObject(BLACK_BRUSH));

	if (IBORDER!=0)
		Rectangle(hdc,r.left,r.top,IBORDER,r.bottom);
	if ((m_inicons*(ICONWIDTH+ICONBORDER)+IBORDER)<r.right)
		Rectangle(hdc,m_inicons*(ICONWIDTH+ICONBORDER)+IBORDER,r.top,r.right,r.bottom);
	for (i=0;i<m_inicons;i++)
	{
		DrawIcon(hdc,i*(ICONWIDTH+ICONBORDER)+IBORDER,0,m_aiconhandles[i][m_astate[i]]);
		Rectangle(hdc,i*(ICONWIDTH+ICONBORDER)+IBORDER+ICONWIDTH,r.top,(i+1)*(ICONWIDTH+ICONBORDER)+IBORDER,r.bottom);
	}

}

void CIconBar::SetIconState(int icon,int state,bool update,bool changeparams)
{
	int i;
	switch (m_aicontype[icon])
	{
	case ITYPEHEAT:
		{
			if ((state==IMOUSEDOWN)&&(m_astate[icon]!=ISELECTED))
			{
				state=ISELECTED;
				for (i=0;i<m_inicons;i++)
					if (m_aicontype[i]==ITYPEWAVE)
						m_astate[i]=INORMAL;
				m_astate[icon]=state;
				if (changeparams)
					ChangeRule(RULEHEAT);
			}
			break;
		}
	case ITYPEWAVE:
		{
			if ((state==IMOUSEDOWN)&&(m_astate[icon]!=ISELECTED))
			{
				state=ISELECTED;
				for (i=0;i<m_inicons;i++)
					if (m_aicontype[i]==ITYPEHEAT)
						m_astate[i]=INORMAL;
				m_astate[icon]=state;
				if (changeparams)
					ChangeRule(RULEWAVE);
			}
			break;
		}
	case ITYPEPAUSE:
		{
			if ((state==IMOUSEDOWN)&&(m_astate[icon]!=IDISABLED))
			{
				if (m_astate[icon]==INORMAL)
					state=ISELECTED;
				else
					state=INORMAL;
			}
			if ((state==INORMAL)||(state==ISELECTED))
			{
				m_astate[icon]=state;
				if (changeparams)
					g_bPaused=(m_astate[icon]==ISELECTED);
			}
			if (state==IDISABLED)
				m_astate[icon]=state;


			break;
		}
	case ITYPEABOUT:
		{
			if (state==IMOUSEDOWN)
			{
				DoAbout();
				SetTimer(m_hwnd,2,ABOUTTIME,NULL);
				state=ISELECTED;
			}
			if ((state==ISELECTED)||(state==INORMAL))
				m_astate[icon]=state;
			break;
		}
	case ITYPEENERGY:
		{
			if ((state==IMOUSEDOWN)&&(m_astate[icon]!=ISELECTED))
			{
				state=ISELECTED;
				for (i=0;i<m_inicons;i++)
					if (m_aicontype[i]==ITYPECOLOUR)
						m_astate[i]=INORMAL;
				m_astate[icon]=state;
				if (changeparams)
				{
					g_aSlider[0].SetTypeandValue(SLIDER_SPEED,g_ruleparams.speed);
					g_aSlider[0].InvalidateCore();
				}
			}
			break;
		}
	case ITYPECOLOUR:
		{
			if ((state==IMOUSEDOWN)&&(m_astate[icon]!=ISELECTED))
			{
				state=ISELECTED;
				for (i=0;i<m_inicons;i++)
					if (m_aicontype[i]==ITYPEENERGY)
						m_astate[i]=INORMAL;
				m_astate[icon]=state;
				if (changeparams)
				{
					g_aSlider[0].SetTypeandValue(SLIDER_CMAP,g_icurrentcmap);
					g_aSlider[0].InvalidateCore();
				}
			}
			break;
		}
	case ITYPEOFF:
		{
			if (state==IMOUSEDOWN){
				SendMessage(GetParent(m_hwnd),WM_CLOSE,0,0);
			}
			break;
		}
	}
	if (update)
	{
		InvalidateRect(m_hwnd,NULL,true);
		UpdateWindow(m_hwnd);
	}
}


void CIconBar::DoClick(int x, int y)
{
	int icon;
	bool hitone=false;
	x=x-IBORDER;
	if (x>=0)
	{
		icon=x/(ICONWIDTH+ICONBORDER);
		if (icon<m_inicons)
			hitone=(x<=(icon*(ICONWIDTH+ICONBORDER)+ICONWIDTH));
	}
	if (hitone)
		SetIconState(icon,IMOUSEDOWN,true,true);

}

int	CIconBar::FindIconFromType(int type)
{
	int i;
	for (i=0;i<m_inicons;i++)
		if (m_aicontype[i]==type)
			return i;
	return -1;
}

LRESULT CALLBACK IBWndProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_CREATE:
			{
			int i;
			RECT r;
			GetClientRect(hWnd,&r);
			i=GetWindowLong(hWnd,GWL_ID);
			g_aIconBar[i-FIRSTIBID].AddIconBar(hWnd);
			return 0;
			}
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC dc=BeginPaint(hWnd,&ps);
				g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].Paint(dc);
				EndPaint(hWnd,&ps);
				return 0;
			}
		case WM_LBUTTONDOWN:
			{
				int x,y;
				x=GET_X_LPARAM(lParam);
				y=GET_Y_LPARAM(lParam);
				g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].DoClick(x,y);
				return 0;
			}
		case WM_TIMER:
			{
				KillTimer(hWnd,2);
				g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].SetIconState(g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].FindIconFromType(ITYPEPAUSE),INORMAL,true,true);
				g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].SetIconState(g_aIconBar[GetWindowLong(hWnd,GWL_ID)-FIRSTIBID].FindIconFromType(ITYPEABOUT),INORMAL,true,true);
				break;
			}

		default:
			return DefWindowProc(hWnd,uMsg,wParam,lParam);
		break;
	}
	 return 0;
}



bool RegisterIconBarClass()
{

	iconbarwc.cbClsExtra=0;
	iconbarwc.cbWndExtra=0;
	iconbarwc.hbrBackground=NULL;
	iconbarwc.hCursor=LoadCursor(NULL,IDC_ARROW);
	iconbarwc.hIcon=NULL;
	iconbarwc.hInstance=g_hinst;
	iconbarwc.lpfnWndProc=IBWndProc;
	iconbarwc.lpszClassName=_T("iconbar");
	iconbarwc.lpszMenuName=0;
	iconbarwc.style=CS_HREDRAW|CS_VREDRAW;

	if(!RegisterClass(&iconbarwc))
	{
		MessageBox(NULL,_T("Couldn't Register Window Class"),
		_T("Window Class Registration Failure"),MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}


bool CreateIconBarWindow(HWND inhWnd,int lpos, int tpos, int xsize, int ysize)
// true = success
{
  
	HWND hwnd=NULL;
	if (g_inexticonbar<NIBS+FIRSTIBID)
		hwnd = CreateWindowEx(NULL,_T("iconbar"),_T("iconbar"),
		WS_CHILD | WS_VISIBLE ,lpos,tpos,xsize,ysize,inhWnd,HMENU(INT64(0)|g_inexticonbar++),
		g_hinst,0);

	if(hwnd == NULL)
	{
		MessageBox(NULL,_T("Failed to Create Window"),_T("Window Creation Failure"),
		MB_OK|MB_ICONERROR);
		return false;
	}

	g_aIconBar[0].AddIcon(0,IDI_HEAT_1,ITYPEHEAT,false);
	g_aIconBar[0].SetIconState(0,IMOUSEDOWN,false,false);
	g_aIconBar[0].AddIcon(1,IDI_WAVE_1,ITYPEWAVE,false);
#ifndef _WIN32_WCE
	g_aIconBar[0].AddIcon(2,IDI_PAUSE_1,ITYPEPAUSE,false);
	g_aIconBar[0].AddIcon(3,IDI_ABOUT_1,ITYPEABOUT,false);
#else
	g_aIconBar[0].AddIcon(2,IDI_ENERGY_1,ITYPEENERGY,false);
	g_aIconBar[0].SetIconState(2,IMOUSEDOWN,false,false);
	g_aIconBar[0].AddIcon(3,IDI_COLOUR_1,ITYPECOLOUR,false);
	g_aIconBar[0].AddIcon(4,IDI_PAUSE_1,ITYPEPAUSE,false);
	g_aIconBar[0].AddIcon(5,IDI_ABOUT_1,ITYPEABOUT,false);
	g_aIconBar[0].AddIcon(6,IDI_OFF_1,ITYPEOFF,false);
#endif

	ShowWindow(hwnd,SW_SHOW);
	UpdateWindow(hwnd);

  return true;
}