//Copyright (C) 2005  Iain Sharp
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//Linking Swell statically or dynamically with other modules is making a
//combined work based on Swell.  Thus, the terms and conditions of the GNU
//General Public License cover the whole combination.
//
//As a special exception, the copyright holders of Swell give you
//permission to integrate Swell and derivative works 
//with multimedia libraries and toolkits similar to
//DirectX and PocketHAL, regardless of the license
//terms of these independent modules, and to copy and distribute the
//resulting combined work under the GPL maintaining this exception.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// For information and contacts for the author go to www.pinedragon.com/swell
//
#define WIN32_LEAN_AND_MEAN

#include <math.h>
#ifndef _WIN32_WCE
	#include <crtdbg.h>
#endif
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "defs.h"
#include "platformdefs.h"
#include "resource.h"
#include "slider.h"

static WNDCLASS sliderwc;
CSlider g_aSlider[NSLIDERS];
int g_inextslider=0;
int g_icurrentcmap=0;



#ifndef max
	#define max(x,y) ((x) > (y) ? (x) : (y))
#endif

#ifdef _WIN32_WCE
	POINT g_aap[2];
	
	BOOL MoveToEx(HDC hdc,int X,int Y,LPPOINT lpPoint)
	{
		g_aap[0].x=X;
		g_aap[0].y=Y;
		return true;
	}

	BOOL LineTo(HDC hdc,int nXEnd,int nYEnd){
		g_aap[1].x=nXEnd;
		g_aap[1].y=nYEnd;
		Polyline(hdc,g_aap,2);
		g_aap[0].x=nXEnd;
		g_aap[0].y=nYEnd;
		return true;
	};

#endif


void Triangle(HDC hdc, int x1, int y1, int x2, int y2, int x3, int y3)
{
	POINT	ap[3];
	ap[0].x=x1;
	ap[0].y=y1;
	ap[1].x=x2;
	ap[1].y=y2;
	ap[2].x=x3;
	ap[2].y=y3;
	Polygon(hdc,ap,3);
}


void	CSlider::GetSMainRect(RECT* r)
{
	GetClientRect(m_hwnd,r);
	if (m_bisvertical)
		InsetRect(r,SLXMARGIN,SLYMARGIN);
	else
		InsetRect(r,SLYMARGIN,SLXMARGIN);
}

void	CSlider::GetSClientRect(RECT* r)
{
	GetClientRect(m_hwnd,r);
}


CSlider::CSlider(){
	m_hwnd=NULL;
	m_iminvalue=0;
	m_imaxvalue=100;
	m_icurrentvalue=0;
	m_bisvertical=false;
	m_islidertype=0;
	m_imousedown=0;
}

CSlider::~CSlider(){
}

int CSlider::GetRelCPos()
{
	RECT r;
	GetSMainRect(&r);
	if (m_bisvertical)
		return int((r.bottom-r.top-1)*(m_icurrentvalue-m_iminvalue)/(m_imaxvalue-m_iminvalue));
	else
		return int((r.right-r.left-1)*(m_icurrentvalue-m_iminvalue)/(m_imaxvalue-m_iminvalue));
}

int CSlider::GetCPos()
{
	RECT r;
	GetSMainRect(&r);
	if (m_bisvertical)
		return r.bottom-1-int((r.bottom-r.top-1)*(m_icurrentvalue-m_iminvalue)/(m_imaxvalue-m_iminvalue));
	else
		return r.left+int((r.right-r.left-1)*(m_icurrentvalue-m_iminvalue)/(m_imaxvalue-m_iminvalue));
}

void CSlider::SetTypeandValue(int type,int value)
{
	m_islidertype=type;
	switch (m_islidertype)
	{
		case SLIDER_SPEED:
			SetRange(MINSPEED,MAXSPEED);
			break;
		case SLIDER_CMAP:
			SetRange(0,g_incmaps-1);
			break;
	}
	m_icurrentvalue=value;
}

void CSlider::AddCSlider(HWND hwnd, bool vertical, int type)
{
	m_hwnd=hwnd;
	m_bisvertical=vertical;
	SetTypeandValue(type,0);

}


void CSlider::SetRange(int min, int max)
{
	m_iminvalue=min;
	m_imaxvalue=max;
	if (m_icurrentvalue>m_imaxvalue)
		m_icurrentvalue=m_imaxvalue;
	if (m_icurrentvalue<m_iminvalue)
		m_icurrentvalue=m_iminvalue;
}

void CSlider::PaintArrow(HDC hdc, int dir)
{
	RECT r;
	GetSMainRect(&r);
	int cl,i0,i1;
	
	if (dir!=m_imousedown)
	{
		SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(127,127,127)));
		SelectObject(hdc,GetStockObject(BLACK_BRUSH));
	}
	else
	{
		SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(255,255,255)));
		SelectObject(hdc,GetStockObject(GRAY_BRUSH));
	}
	if (m_bisvertical)
	{
		i0=(r.right+r.left-SARRWIDTH)/2;
		i1=i0+SARRWIDTH-1;
		cl=(i0+i1)/2;
		if (dir<=0)
			Triangle(hdc,i0,r.bottom+SARRBORDER-1,i1,r.bottom+SARRBORDER-1,cl,r.bottom+SARRBORDER+SARRHEIGHT-1);
		else
			Triangle(hdc,i0,r.top-SARRBORDER,i1,r.top-SARRBORDER,cl,r.top-SARRBORDER-SARRHEIGHT);
	}
	else
	{
		i0=(r.top+r.bottom-SARRWIDTH)/2;
		i1=i0+SARRWIDTH-1;
		cl=(i0+i1)/2;
		if (dir<=0)
			Triangle(hdc,r.left-SARRBORDER,i0,r.left-SARRBORDER,i1,r.left-SARRBORDER-SARRHEIGHT,cl);
		else
			Triangle(hdc,r.right+SARRBORDER-1,i0,r.right+SARRBORDER-1,i1,r.right+SARRBORDER+SARRHEIGHT-1,cl);
	}
	DeleteObject(SelectObject(hdc,GetStockObject(WHITE_PEN)));
}

void CSlider::PaintSpeed(HDC hdc){
	RECT r;
	int i,size,maxi;
	DWORD c;

	GetSMainRect(&r);


	size = m_bisvertical?r.bottom-r.top:r.right-r.left;
	maxi=GetRelCPos();
	SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(0,0,0)));
	for (i=0; i<=maxi;i++)
	{
		c=g_aMyCmap[(i*255)/size];
		DeleteObject(SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(RG(c),GG(c),BG(c)))));
		if (m_bisvertical)
		{
			MoveToEx(hdc,r.left,r.bottom-1-i,NULL);
			LineTo(hdc,r.right,r.bottom-1-i);
		}
		else
		{
			MoveToEx(hdc,r.left+i,r.top,NULL);
			LineTo(hdc,r.left+i,r.bottom);
		}
	}
	DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN)));
	SelectObject(hdc,GetStockObject(BLACK_BRUSH));
	if (m_bisvertical)
			Rectangle(hdc,r.left,r.bottom-maxi-1,r.right,r.top);
		else
			Rectangle(hdc,r.left+maxi,r.top,r.right,r.bottom);
}

void CSlider::PaintCmap(HDC hdc){
	RECT r;
	int i,size;
	DWORD c;

	GetSMainRect(&r);
	SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(0,0,0)));
	size = m_bisvertical?r.bottom-r.top:r.right-r.left;
	for (i=0; i<size;i++)
	{
		c=g_aSliderCmap[(i*255)/size];
		DeleteObject(SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(RG(c),GG(c),BG(c)))));
		if (m_bisvertical)
		{
			MoveToEx(hdc,r.left,r.bottom-1-i,NULL);
			LineTo(hdc,r.right,r.bottom-1-i);
		}
		else
		{
			MoveToEx(hdc,r.left+i,r.top,NULL);
			LineTo(hdc,r.left+i,r.bottom);
		}
	}
	DeleteObject(SelectObject(hdc,GetStockObject(WHITE_PEN)));
}

void CSlider::Paint(HDC hdc){
	RECT r,r2,r3;
	int i;


	GetClientRect(m_hwnd,&r);
	GetSMainRect(&r2);
	r3=r2;
	InflateRect(&r3,1,1);
	SelectObject(hdc,GetStockObject(BLACK_BRUSH));
	SelectObject(hdc,GetStockObject(BLACK_PEN));
	if (m_bisvertical)
		{
			Rectangle(hdc,r.left,r.top,r.right,r3.top);
			Rectangle(hdc,r.left,r3.bottom,r.right,r.bottom);
		}
	else
		{
			Rectangle(hdc,r.left,r.top,r3.left,r.bottom);
			Rectangle(hdc,r3.right,r.top,r.right,r.bottom);
		}

	SelectObject(hdc,CreatePen(PS_SOLID,0,RGB(127,127,127)));
	SelectObject(hdc,GetStockObject(NULL_BRUSH));

	Rectangle(hdc,r3.left,r3.top,r3.right,r3.bottom);
	DeleteObject(SelectObject(hdc,GetStockObject(WHITE_PEN)));
	PaintArrow(hdc,-1);
	PaintArrow(hdc,1);
	switch (m_islidertype)
	{
		case SLIDER_SPEED:
			PaintSpeed(hdc);
			break;
		case SLIDER_CMAP:
			PaintCmap(hdc);
			break;
	}
	SelectObject(hdc,GetStockObject(BLACK_PEN));
	SelectObject(hdc,GetStockObject(BLACK_BRUSH));
	i=GetCPos();
	if (m_bisvertical)
	{
		Rectangle(hdc,r.left,r3.top,r3.left,r3.bottom);
		Rectangle(hdc,r3.right,r3.top,r.right,r3.bottom);
		SelectObject(hdc,GetStockObject(WHITE_PEN));
		SelectObject(hdc,GetStockObject(WHITE_BRUSH));
		Rectangle(hdc,SLXBORDER,i-SCHEIGHT,SLXBORDER+SCWIDTH,i+1+SCHEIGHT);
		Rectangle(hdc,r.right-SLXBORDER-SCWIDTH,i-SCHEIGHT,r.right-SLXBORDER,i+1+SCHEIGHT);
		MoveToEx(hdc,SLXBORDER,i,NULL);
		LineTo(hdc,r.right-SLXBORDER-SCWIDTH,i);
	}
	else
	{
		Rectangle(hdc,r3.left,r.top,r3.right,r3.top);
		Rectangle(hdc,r3.left,r3.bottom,r3.right,r.bottom);
		SelectObject(hdc,GetStockObject(WHITE_PEN));
		SelectObject(hdc,GetStockObject(WHITE_BRUSH));
		Rectangle(hdc,i-SCHEIGHT,SLXBORDER,i+1+SCHEIGHT,SLXBORDER+SCWIDTH);
		Rectangle(hdc,i-SCHEIGHT,r.bottom-SLXBORDER-SCWIDTH,i+1+SCHEIGHT,r.bottom-SLXBORDER);
		MoveToEx(hdc,i,SLXBORDER,NULL);
		LineTo(hdc,i,r.bottom-SLXBORDER-SCWIDTH);
	}
	
}

static int clip(int x,int min,int max)
{
	if(x<min)
		return min;
	else
		if (x>max)
			return max;
		else
			return x;
}

void CSlider::DoUnClick()
{
	int i;
	if (m_imousedown!=0)
	{
		i=m_imousedown;
		m_imousedown=0;
		HDC hdc=GetDC(m_hwnd);
		PaintArrow(hdc,i);
		ReleaseDC(m_hwnd,hdc);
	}
	InternalSetValue(m_icurrentvalue,true);
}

void CSlider::InternalSetValue(int n,bool isfinal){
	n=clip(n,m_iminvalue,m_imaxvalue);
	if ((m_icurrentvalue!=n)||(isfinal&&(m_istoredvalue!=n)))
		{
		m_icurrentvalue=n;
		if (isfinal)
			m_istoredvalue=n;
		int j;
		switch (m_islidertype)
			{
				case SLIDER_SPEED:
					g_ruleparams.speed=n;
					InvalidateCore();
					break;
				case SLIDER_CMAP:
					{
						makeslidermap(n);
						maketargetmap(n);
						g_icurrentcmap=n;
						for (j=0;j<g_inextslider;j++)
								g_aSlider[j].InvalidateCore();
					break;
					}
			}
	}
}

void CSlider::UpdateFromMouse(int dd,bool first){
	int incr,dist;
	RECT r;
	if (first)
		m_istoredvalue=m_icurrentvalue;
	else
	{
		GetSMainRect(&r);
		dist=(m_bisvertical)?r.bottom-r.top:r.right-r.left;
		switch (m_islidertype)
		{
			case SLIDER_SPEED:
				incr=dd*(m_imaxvalue-m_iminvalue)/dist;
				break;
			case SLIDER_CMAP:
				incr=dd/SMINMOUSEJOG;
				break;
		}
		InternalSetValue(incr+m_istoredvalue,false);
	}
}

int CSlider::DoClickArrow(int dir)
{
	int incr;
	bool update;
	switch (m_islidertype){
		case SLIDER_CMAP:
			incr=dir;
			break;
		default:
			incr=dir*max(1,int((m_imaxvalue-m_iminvalue)/200));
	}
	update=(m_imousedown!=dir);
	m_imousedown=dir;
	if (update)
	{
		HDC hdc=GetDC(m_hwnd);
		PaintArrow(hdc,dir);
		ReleaseDC(m_hwnd,hdc);
	}
	if (m_islidertype==SLIDER_SPEED)
		InternalSetValue(m_icurrentvalue+incr,true);
	return m_icurrentvalue+incr;
}

void CSlider::Invalidate()
{
	InvalidateRect(m_hwnd,NULL,true);
	UpdateWindow(m_hwnd);
}

void CSlider::InvalidateCore()
{
	RECT rinv,r;
	GetClientRect(m_hwnd,&r);
	if (m_bisvertical)
		InsetRect(&r,0,SLYMARGIN-SCHEIGHT);
	else
		InsetRect(&r,SLYMARGIN-SCHEIGHT,0);
	InvalidateRect(m_hwnd,&r,true);
	UpdateWindow(m_hwnd);
}

// Returns true if clicked in an arrow, false otherwise
bool CSlider::DoClick(int x, int y,bool firstclick)
{
	RECT r;
	GetSMainRect(&r);
	int n,pos;
	n=m_icurrentvalue;
	if (firstclick)
		m_istoredvalue=m_icurrentvalue;
	if (m_bisvertical)
	{
		if (y>=r.bottom+SARRBORDER/2)
			pos=-1;
		else
			if (y<=r.top-SARRBORDER/2)
				pos=+1;
			else
			{
				if (firstclick||(m_imousedown==0))
					n =int(0.5+double((r.bottom-y)*(m_imaxvalue-m_iminvalue))/(r.bottom-r.top))+m_iminvalue;
				pos=0;
			}
	}
	else
	{
		if (x>=r.right+SARRBORDER/2)
			pos=1;
		else
			if (x<=r.left-SARRBORDER/2)
				pos=-1;
			else
			{
				if (firstclick||(m_imousedown==0))
					n= int(0.5+double((x-r.left)*(m_imaxvalue-m_iminvalue))/(r.right-r.left))+m_iminvalue;
				pos=0;
			}
	}
	if (firstclick||(pos==m_imousedown))
	{
		if (pos==-1)
			n=DoClickArrow(-1);
		if (pos==1)
			n=DoClickArrow(1);
		if (pos==0)
			m_imousedown=0;
	}
	n=clip(n,m_iminvalue,m_imaxvalue);
	if (n!=m_icurrentvalue)
	{
		switch (m_islidertype)
		{
			case SLIDER_SPEED:
				InternalSetValue(n,true);
				break;
			case SLIDER_CMAP:
				{
					m_icurrentvalue=n;
					makeslidermap(n);
					InvalidateCore();
				break;
				}
		}

	}

	return (pos!=0);
}

LRESULT CALLBACK SWndProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
	static bool lastarrowclick=false;
	static int lastx,lasty;
	switch(uMsg)
	{
		case WM_CREATE:
			{
			int i;
			RECT r;
			GetClientRect(hWnd,&r);
			i=GetWindowLong(hWnd,GWL_ID);
			g_aSlider[i].AddCSlider(hWnd,(r.bottom-r.top)>(r.right-r.left),i);
			return 0;
			}
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC dc=BeginPaint(hWnd,&ps);
				g_aSlider[GetWindowLong(hWnd,GWL_ID)].Paint(dc);
				EndPaint(hWnd,&ps);
				return 0;
			}
		case WM_RBUTTONDOWN:
		case WM_LBUTTONDOWN:
			{
				lastx=GET_X_LPARAM(lParam);
				lasty=GET_Y_LPARAM(lParam);
				lastarrowclick=g_aSlider[GetWindowLong(hWnd,GWL_ID)].DoClick(lastx,lasty,true);
				if (lastarrowclick)
					SetTimer(hWnd,1,SFIRSTREPEATTIME,NULL);
				SetCapture(hWnd);
				return 0;
			}
		case WM_RBUTTONUP:
		case WM_LBUTTONUP:
			{
				ReleaseCapture();
				if (lastarrowclick)
				{
					lastarrowclick=false;
					KillTimer(hWnd,1);
				}
				g_aSlider[GetWindowLong(hWnd,GWL_ID)].DoUnClick();
				return 0;
			}
		case WM_MOUSEMOVE:
			{if(((wParam & MK_LBUTTON)||(wParam & MK_RBUTTON))&&(!lastarrowclick))
			{
			g_aSlider[GetWindowLong(hWnd,GWL_ID)].DoClick(GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),false);
			}
			return 0;
			}
		case WM_TIMER:
			{
				SetTimer(hWnd,1,SREPEATTIME,NULL);
				lastarrowclick=g_aSlider[GetWindowLong(hWnd,GWL_ID)].DoClick(lastx,lasty,false);
			}
		default:
			return DefWindowProc(hWnd,uMsg,wParam,lParam);
		break;
	}
	 return 0;
}



bool RegisterSliderClass()
{

	sliderwc.cbClsExtra=0;
	sliderwc.cbWndExtra=0;
	sliderwc.hbrBackground=NULL;
	sliderwc.hCursor=LoadCursor(NULL,IDC_ARROW);
	sliderwc.hIcon=NULL;
	sliderwc.hInstance=g_hinst;
	sliderwc.lpfnWndProc=SWndProc;
	sliderwc.lpszClassName=_T("slider");
	sliderwc.lpszMenuName=0;
	sliderwc.style=CS_HREDRAW|CS_VREDRAW;

	if(!RegisterClass(&sliderwc))
	{
		MessageBox(NULL,_T("Couldn't Register Window Class"),
		_T("Window Class Registration Failure"),MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}


bool CreateSliderWindow(HWND inhWnd,int lpos, int tpos, int xsize, int ysize)
// true = success
{
  
	HWND hwnd=NULL;
	if (g_inextslider<NSLIDERS)
		hwnd = CreateWindowEx(NULL,_T("slider"),_T("slider"),
		WS_CHILD | WS_VISIBLE ,lpos,tpos,xsize,ysize,inhWnd,HMENU(INT64(0)|g_inextslider++),
		g_hinst,0);

	if(hwnd == NULL)
	{
		MessageBox(NULL,_T("Failed to Create Window"),_T("Window Creation Failure"),
		MB_OK|MB_ICONERROR);
		return false;
	}

	ShowWindow(hwnd,SW_SHOW);
	UpdateWindow(hwnd);

  return true;
}